/*****************************************************
Chip type           : AT90S2313
Clock frequency     : 4,000000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 32
*****************************************************/

#include <90s2313.h>


#include <stdlib.h>
#include <delay.h>

//       
#define OUT PORTB
#define MOTOR_F  7
#define MOTOR_B  6
#define TURN_R   5
#define TURN_L   4

#define IN PIND
#define LIGHT_R  0
#define LIGHT_L  1
#define BUMPER_F 2
#define BUMPER_B 3

//   
enum {STOP, F, FR, FL, B, BR, BL};

//------------------------------------------------------------------------------
//  t  10ms
//------------------------------------------------------------------------------
void Delay_10ms(unsigned char t) 
{char i;
  for(i=0;i<t;i++) delay_ms(10);
}

//      
//     
unsigned char p[7][7] = 
{14,	43,	57,	71,	86,	93,	100,
 7,	43,	71,	100,	100,	100,	100,
 7,	50,	93,	100,	100,	100,	100,
 7,	50,	57,	100,	100,	100,	100,
 29,	29,	29,	29,	57,	79,	100,
 36,	36,	36,	36,	71,	93,	100,
 36,	36,	36,	36,	71,	79,	100};

//   
unsigned char this_move; 

//------------------------------------------------------------------------------
//        
//------------------------------------------------------------------------------
void go(unsigned char direction){

  switch (direction) {
  case STOP:
    OUT.MOTOR_F=0;
    OUT.MOTOR_B=0;
    OUT.TURN_R=0;
    OUT.TURN_L=0;
    break;

  case F:
    OUT.MOTOR_F=1;
    OUT.MOTOR_B=0;
    OUT.TURN_R=0;
    OUT.TURN_L=0;
    break;

  case FR:
    OUT.MOTOR_F=1;
    OUT.MOTOR_B=0;
    OUT.TURN_R=1;
    OUT.TURN_L=0;
    break;

  case FL:
    OUT.MOTOR_F=1;
    OUT.MOTOR_B=0;
    OUT.TURN_R=0;
    OUT.TURN_L=1;
    break;

  case B:
    OUT.MOTOR_F=0;
    OUT.MOTOR_B=1;
    OUT.TURN_R=0;
    OUT.TURN_L=0;
    break;

  case BR:
    OUT.MOTOR_F=0;
    OUT.MOTOR_B=1;
    OUT.TURN_R=1;
    OUT.TURN_L=0;
    break;

  case BL:
    OUT.MOTOR_F=0;
    OUT.MOTOR_B=1;
    OUT.TURN_R=0;
    OUT.TURN_L=1;
    break;
  }
}

//------------------------------------------------------------------------------
//         
//------------------------------------------------------------------------------
unsigned char next_move(void){
   unsigned char pp, i;

   pp = rand()/327;     //    0..99
   for (i=0;i<7;i++){   //     
      if (p[this_move][i] > pp) break;
   }
   this_move = i;       //      
   return(i);
}

//------------------------------------------------------------------------------
//      (INT0 = PD2)
//------------------------------------------------------------------------------
interrupt [EXT_INT0] void ext_int0_isr(void)
{
   if(this_move==FR) go(BL);
   if(this_move==FL) go(BR);
   else go(B);
   Delay_10ms(250);    //    2.5  2 
   Delay_10ms(250);
   this_move=B;
}

//------------------------------------------------------------------------------
//      (INT1 = PD3)
//------------------------------------------------------------------------------
interrupt [EXT_INT1] void ext_int1_isr(void)
{
   if(this_move==BR) go(FL);
   if(this_move==BL) go(FR);
   else go(F);
   Delay_10ms(250);    //    2.5  2 
   Delay_10ms(250);
   this_move=F;
}

//------------------------------------------------------------------------------
// " "
//------------------------------------------------------------------------------
unsigned char walk(void){
   //    " " 
   //        
   while((IN.LIGHT_R) && (IN.LIGHT_L)){
       go(next_move());   //     
       Delay_10ms(250);   //     2.5 
   }
   //      , 
   //         
   while((IN.LIGHT_R==0) || (IN.LIGHT_L==0)){
       if((IN.LIGHT_R==0) && (IN.LIGHT_L==0)) go(F);
       else if(IN.LIGHT_R==0) go(FR);
       else if(IN.LIGHT_L==0) go(FL);
   }

   return(0);
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void main(void)
{
  DDRB  = 0xff;  //     B  
  PORTB = 0x00;  //      

  DDRD  = 0x00;  //     D  
  PORTD = 0xff;  //    

  // External Interrupt(s) initialization
  // INT0: On
  // INT0 Mode: Falling Edge
  // INT1: On
  // INT1 Mode: Falling Edge
  GIMSK=0xC0;
  MCUCR=0x0A;
  GIFR=0xC0;

  //  
  #asm("sei")

  //   
  while(1) walk();

}


